"use strict";
this.name        = "tracker_beacon.js";
this.author      = "Thargoid";
this.copyright   = "CC BY-NC-SA 3.0";
this.description = "Beacon script.";
this.version     = "1.04";

this.$tracked;
this.$trackedName;


//
// Event Handlers
//

//--------------------------------------------------------------------------------------------//
this.shipSpawned = function _shipSpawned() { 
    if (!this.ship || !this.ship.isValid || !this.ship.target) {
        log(this.name, "Incomplete tracker beacon:"+this.ship+(this.ship ? ", "+this.ship.isValid+", "+this.ship.target : ""));
        return;
    }
    log(this.name, "Tracker for "+this.ship.target.displayName+" spawned, adding collision exception and setting up beacon label");
    this.ship.addCollisionException(this.ship.target);
    this.$tracked = this.ship.target;
    this.$trackedName= this.ship.target.displayName;
    this.ship.beaconCode = "Tracker: "+this.$trackedName;
    this.ship.beaconLabel = "Tracker: "+this.$trackedName;
    this.callbackID = addFrameCallback(this.repositionTracker.bind(this)); 
}

//--------------------------------------------------------------------------------------------//
this.shipWillEnterWitchspace = function _shipWillEnterWitchspace() {
    log(this.name, "Tracker for " + this.$trackedName + " entering wormhole");
}

//--------------------------------------------------------------------------------------------//
this.shipDied = function _shipDied(whom, why) { 
    if (why && why === "scrape damage" && whom && this.ship && this.$tracked && whom === this.$tracked) {
        // in cases where the tag does manage to collide with the tracked entity, respawn the tag.
        var Tag = this.ship.spawnOne("tracker_beacon");
        Tag.target = this.$tracked;
        log(this.name, "Tracker for "+this.$trackedName+" collided with ship and was destroyed, creating a new one");
    } else {
        log(this.name, "Tracker for "+this.$trackedName+" was destroyed by '"+whom+"' with '"+why+"'");
    }
}


//
// Internal Functions
//

//--------------------------------------------------------------------------------------------//
this.repositionTracker = function _repositionTracker() {
    if (!this.ship || !this.ship.isValid || !this.$tracked || !this.$tracked.isValid) {
        // something has gone wrong
        if (this.ship && this.ship.isValid)
            // tracker is OK, but the tracked ship is not
            log(this.name, this.$trackedName+" is no more, removing tracker (tracker target:"+this.ship.target+")");
        else if (this.ship)
            // tracker exists but is not valid anymore
            log(this.name, "Tracker for "+this.$trackedName+" is not valid anymore, removing");
        else
            // tracker does not exist anymore but this FCB is still being called
            log(this.name, "Tracker undefined:"+this.ship+", tracked:"+this.$trackedName);
        if (this.callbackID) {
            removeFrameCallback(this.callbackID);
            delete this.callbackID;
        }
        this.ship.remove();
        return;
    }
        
    this.ship.position = this.$tracked.position
}

    
